@@ -4,7 +4,7 @@
 // Purpose:     Part of the widgets sample showing wxSlider
 // Author:      Vadim Zeitlin
 // Created:     16.04.01
-// Id:          $Id: slider.cpp,v 1.27 2005/08/28 08:54:55 MBN Exp $
+// Id:          $Id: slider.cpp 43755 2006-12-03 13:43:44Z VZ $
 // Copyright:   (c) 2001 Vadim Zeitlin
 // License:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
@@ -60,12 +60,16 @@ enum
     SliderPage_Clear,
     SliderPage_SetValue,
     SliderPage_SetMinAndMax,
+    SliderPage_SetLineSize,
+    SliderPage_SetPageSize,
     SliderPage_SetTickFreq,
     SliderPage_SetThumbLen,
     SliderPage_CurValueText,
     SliderPage_ValueText,
     SliderPage_MinText,
     SliderPage_MaxText,
+    SliderPage_LineSizeText,
+    SliderPage_PageSizeText,
     SliderPage_TickFreqText,
     SliderPage_ThumbLenText,
     SliderPage_RadioSides,
@@ -89,10 +93,14 @@ enum
 class SliderWidgetsPage : public WidgetsPage
 {
 public:
-    SliderWidgetsPage(wxBookCtrlBase *book, wxImageList *imaglist);
+    SliderWidgetsPage(WidgetsBookCtrl *book, wxImageList *imaglist);
     virtual ~SliderWidgetsPage(){};
 
     virtual wxControl *GetWidget() const { return m_slider; }
+    virtual void RecreateWidget() { CreateSlider(); }
+
+    // lazy creation of the content
+    virtual void CreateContent();
 
 protected:
     // event handlers
@@ -100,6 +108,8 @@ protected:
     void OnButtonClear(wxCommandEvent& event);
     void OnButtonSetValue(wxCommandEvent& event);
     void OnButtonSetMinAndMax(wxCommandEvent& event);
+    void OnButtonSetLineSize(wxCommandEvent& event);
+    void OnButtonSetPageSize(wxCommandEvent& event);
     void OnButtonSetTickFreq(wxCommandEvent& event);
     void OnButtonSetThumbLen(wxCommandEvent& event);
 
@@ -109,6 +119,8 @@ protected:
 
     void OnUpdateUIValueButton(wxUpdateUIEvent& event);
     void OnUpdateUIMinMaxButton(wxUpdateUIEvent& event);
+    void OnUpdateUILineSize(wxUpdateUIEvent& event);
+    void OnUpdateUIPageSize(wxUpdateUIEvent& event);
     void OnUpdateUITickFreq(wxUpdateUIEvent& event);
     void OnUpdateUIThumbLen(wxUpdateUIEvent& event);
     void OnUpdateUIRadioSides(wxUpdateUIEvent& event);
@@ -124,6 +136,12 @@ protected:
     // (re)create the slider
     void CreateSlider();
 
+    // set the line size from the text field value
+    void DoSetLineSize();
+
+    // set the page size from the text field value
+    void DoSetPageSize();
+
     // set the tick frequency from the text field value
     void DoSetTickFreq();
 
@@ -156,6 +174,8 @@ protected:
     wxTextCtrl *m_textValue,
                *m_textMin,
                *m_textMax,
+               *m_textLineSize,
+               *m_textPageSize,
                *m_textTickFreq,
                *m_textThumbLen;
 
@@ -172,14 +192,17 @@ BEGIN_EVENT_TABLE(SliderWidgetsPage, WidgetsPage)
     EVT_BUTTON(SliderPage_Reset, SliderWidgetsPage::OnButtonReset)
     EVT_BUTTON(SliderPage_SetValue, SliderWidgetsPage::OnButtonSetValue)
     EVT_BUTTON(SliderPage_SetMinAndMax, SliderWidgetsPage::OnButtonSetMinAndMax)
+    EVT_BUTTON(SliderPage_SetLineSize, SliderWidgetsPage::OnButtonSetLineSize)
+    EVT_BUTTON(SliderPage_SetPageSize, SliderWidgetsPage::OnButtonSetPageSize)
     EVT_BUTTON(SliderPage_SetTickFreq, SliderWidgetsPage::OnButtonSetTickFreq)
     EVT_BUTTON(SliderPage_SetThumbLen, SliderWidgetsPage::OnButtonSetThumbLen)
 
     EVT_UPDATE_UI(SliderPage_SetValue, SliderWidgetsPage::OnUpdateUIValueButton)
     EVT_UPDATE_UI(SliderPage_SetMinAndMax, SliderWidgetsPage::OnUpdateUIMinMaxButton)
+    EVT_UPDATE_UI(SliderPage_SetLineSize, SliderWidgetsPage::OnUpdateUILineSize)
+    EVT_UPDATE_UI(SliderPage_SetPageSize, SliderWidgetsPage::OnUpdateUIPageSize)
     EVT_UPDATE_UI(SliderPage_SetTickFreq, SliderWidgetsPage::OnUpdateUITickFreq)
     EVT_UPDATE_UI(SliderPage_SetThumbLen, SliderWidgetsPage::OnUpdateUIThumbLen)
-    EVT_UPDATE_UI(SliderPage_TickFreqText, SliderWidgetsPage::OnUpdateUITickFreq)
     EVT_UPDATE_UI(SliderPage_RadioSides, SliderWidgetsPage::OnUpdateUIRadioSides)
     EVT_UPDATE_UI(SliderPage_BothSides, SliderWidgetsPage::OnUpdateUIBothSides)
 
@@ -197,14 +220,18 @@ END_EVENT_TABLE()
 // implementation
 // ============================================================================
 
-IMPLEMENT_WIDGETS_PAGE(SliderWidgetsPage, _T("Slider"));
+#if defined(__WXUNIVERSAL__)
+    #define FAMILY_CTRLS UNIVERSAL_CTRLS
+#else
+    #define FAMILY_CTRLS NATIVE_CTRLS
+#endif
 
-SliderWidgetsPage::SliderWidgetsPage(wxBookCtrlBase *book,
+IMPLEMENT_WIDGETS_PAGE(SliderWidgetsPage, _T("Slider"), FAMILY_CTRLS );
+
+SliderWidgetsPage::SliderWidgetsPage(WidgetsBookCtrl *book,
                                      wxImageList *imaglist)
-                  : WidgetsPage(book)
+                  : WidgetsPage(book, imaglist, slider_xpm)
 {
-    imaglist->Add(wxBitmap(slider_xpm));
-
     // init everything
     m_min = 0;
     m_max = 100;
@@ -218,7 +245,10 @@ SliderWidgetsPage::SliderWidgetsPage(wxBookCtrlBase *book,
 
     m_slider = (wxSlider *)NULL;
     m_sizerSlider = (wxSizer *)NULL;
+}
 
+void SliderWidgetsPage::CreateContent()
+{
     wxSizer *sizerTop = new wxBoxSizer(wxHORIZONTAL);
 
     // left pane
@@ -282,6 +312,20 @@ SliderWidgetsPage::SliderWidgetsPage(wxBookCtrlBase *book,
 
     sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
 
+    sizerRow = CreateSizerWithTextAndButton(SliderPage_SetLineSize,
+                                            _T("Li&ne size"),
+                                            SliderPage_LineSizeText,
+                                            &m_textLineSize);
+
+    sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
+
+    sizerRow = CreateSizerWithTextAndButton(SliderPage_SetPageSize,
+                                            _T("P&age size"),
+                                            SliderPage_PageSizeText,
+                                            &m_textPageSize);
+
+    sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
+
     sizerRow = CreateSizerWithTextAndButton(SliderPage_SetTickFreq,
                                             _T("Tick &frequency"),
                                             SliderPage_TickFreqText,
@@ -306,6 +350,9 @@ SliderWidgetsPage::SliderWidgetsPage(wxBookCtrlBase *book,
     Reset();
     CreateSlider();
 
+    m_textLineSize->SetValue(wxString::Format(_T("%d"), m_slider->GetLineSize()));
+    m_textPageSize->SetValue(wxString::Format(_T("%d"), m_slider->GetPageSize()));
+
     // the 3 panes panes compose the window
     sizerTop->Add(sizerLeft, 0, wxGROW | (wxALL & ~wxLEFT), 10);
     sizerTop->Add(sizerMiddle, 0, wxGROW | wxALL, 10);
@@ -313,8 +360,6 @@ SliderWidgetsPage::SliderWidgetsPage(wxBookCtrlBase *book,
 
     // final initializations
     SetSizer(sizerTop);
-
-    sizerTop->Fit(this);
 }
 
 // ----------------------------------------------------------------------------
@@ -333,7 +378,7 @@ void SliderWidgetsPage::Reset()
 
 void SliderWidgetsPage::CreateSlider()
 {
-    int flags = 0;
+    int flags = ms_defaultFlags;
 
     if ( m_chkInverse->GetValue() )
     {
@@ -423,6 +468,42 @@ void SliderWidgetsPage::CreateSlider()
     m_sizerSlider->Layout();
 }
 
+void SliderWidgetsPage::DoSetLineSize()
+{
+    long lineSize;
+    if ( !m_textLineSize->GetValue().ToLong(&lineSize) )
+    {
+        wxLogWarning(_T("Invalid slider line size"));
+
+        return;
+    }
+
+    m_slider->SetLineSize(lineSize);
+
+    if ( m_slider->GetLineSize() != lineSize )
+    {
+        wxLogWarning(_T("Invalid line size in slider."));
+    }
+}
+
+void SliderWidgetsPage::DoSetPageSize()
+{
+    long pageSize;
+    if ( !m_textPageSize->GetValue().ToLong(&pageSize) )
+    {
+        wxLogWarning(_T("Invalid slider page size"));
+
+        return;
+    }
+
+    m_slider->SetPageSize(pageSize);
+
+    if ( m_slider->GetPageSize() != pageSize )
+    {
+        wxLogWarning(_T("Invalid page size in slider."));
+    }
+}
+
 void SliderWidgetsPage::DoSetTickFreq()
 {
     long freq;
@@ -460,6 +541,16 @@ void SliderWidgetsPage::OnButtonReset(wxCommandEvent& WXUNUSED(event))
     CreateSlider();
 }
 
+void SliderWidgetsPage::OnButtonSetLineSize(wxCommandEvent& WXUNUSED(event))
+{
+    DoSetLineSize();
+}
+
+void SliderWidgetsPage::OnButtonSetPageSize(wxCommandEvent& WXUNUSED(event))
+{
+    DoSetPageSize();
+}
+
 void SliderWidgetsPage::OnButtonSetTickFreq(wxCommandEvent& WXUNUSED(event))
 {
     DoSetTickFreq();
@@ -514,6 +605,20 @@ void SliderWidgetsPage::OnUpdateUIValueButton(wxUpdateUIEvent& event)
     event.Enable( m_textValue->GetValue().ToLong(&val) && IsValidValue(val) );
 }
 
+void SliderWidgetsPage::OnUpdateUILineSize(wxUpdateUIEvent& event)
+{
+    long lineSize;
+    event.Enable( m_textLineSize->GetValue().ToLong(&lineSize) &&
+            (lineSize > 0) && (lineSize <= m_max - m_min) );
+}
+
+void SliderWidgetsPage::OnUpdateUIPageSize(wxUpdateUIEvent& event)
+{
+    long pageSize;
+    event.Enable( m_textPageSize->GetValue().ToLong(&pageSize) &&
+            (pageSize > 0) && (pageSize <= m_max - m_min) );
+}
+
 void SliderWidgetsPage::OnUpdateUITickFreq(wxUpdateUIEvent& event)
 {
     long freq;
@@ -562,11 +667,11 @@ void SliderWidgetsPage::OnUpdateUIRadioSides(wxUpdateUIEvent& event)
 
 void SliderWidgetsPage::OnUpdateUIBothSides(wxUpdateUIEvent& event)
 {
-#if defined(__WIN95__) || defined(__WXUNIVERSAL__)
+#if defined(__WXMSW__) || defined(__WXUNIVERSAL__)
     event.Enable( m_chkTicks->GetValue() );
 #else
     event.Enable( false );
-#endif // defined(__WIN95__) || defined(__WXUNIVERSAL__)
+#endif // defined(__WXMSW__) || defined(__WXUNIVERSAL__)
 }
 
 void SliderWidgetsPage::OnSlider(wxScrollEvent& event)
@@ -606,7 +711,7 @@ void SliderWidgetsPage::OnSlider(wxScrollEvent& event)
 
     static int s_numSliderEvents = 0;
 
-    wxLogMessage(wxT("Slider event #%d: %s (pos = %d, int value = %ld)"),
+    wxLogMessage(wxT("Slider event #%d: %s (pos = %d, int value = %d)"),
                  s_numSliderEvents++,
                  eventNames[index],
                  event.GetPosition(),
