@@ -3,7 +3,7 @@
 // Purpose:     wxScreenDC implementation
 // Author:      Vaclav Slavik
 // Created:     2006-08-16
-// RCS-ID:      $Id: dcscreen.cpp 43489 2006-11-18 13:17:35Z VS $
+// RCS-ID:      $Id: dcscreen.cpp 54747 2008-07-21 16:58:06Z VZ $
 // Copyright:   (c) 2006 REA Elektronik GmbH
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -31,7 +31,7 @@
 // wxScreenDC
 //-----------------------------------------------------------------------------
 
-#warning "FIXME: this doesn't work (neither single app nor multiapp core)
+#warning "FIXME: this doesn't work (neither single app nor multiapp core)"
 // FIXME: maybe use a subsurface as well?
 
 IMPLEMENT_DYNAMIC_CLASS(wxScreenDC, wxDC)
