@@ -2,7 +2,7 @@
 // Name:        src/gtk/fontdlg.cpp
 // Purpose:     wxFontDialog
 // Author:      Robert Roebling
-// Id:          $Id: fontdlg.cpp 41597 2006-10-03 16:01:18Z PC $
+// Id:          $Id: fontdlg.cpp 61219 2009-06-27 22:02:51Z VZ $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -108,8 +108,14 @@ bool wxFontDialog::DoCreate(wxWindow *parent)
     m_widget = gtk_font_selection_dialog_new( wxGTK_CONV( m_message ) );
 
     if (parent)
-        gtk_window_set_transient_for(GTK_WINDOW(m_widget),
-                                     GTK_WINDOW(parent->m_widget));
+    {
+        GtkWidget *gtktlw = gtk_widget_get_toplevel(parent->m_widget);
+        if ( gtktlw )
+        {
+            gtk_window_set_transient_for(GTK_WINDOW(m_widget),
+                                         GTK_WINDOW(gtktlw));
+        }
+    }
 
     GtkFontSelectionDialog *sel = GTK_FONT_SELECTION_DIALOG(m_widget);
 
