@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: dc.cpp,v 1.215 2006/11/19 13:31:15 BIW Exp $
+// RCS-ID:      $Id: dc.cpp 63769 2010-03-28 22:34:08Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -209,7 +209,7 @@ public:
     wxOnceOnlyDLLLoader(const wxChar *dllName)
         : m_dllName(dllName)
     {
-    };
+    }
 
 
     // return the symbol with the given name or NULL if the DLL not loaded
@@ -676,9 +676,12 @@ void wxDC::DoDrawArc(wxCoord x1, wxCoord y1,
                      wxCoord x2, wxCoord y2,
                      wxCoord xc, wxCoord yc)
 {
+    double dx = xc - x1;
+    double dy = yc - y1;
+    wxCoord r = (wxCoord)sqrt(dx*dx + dy*dy);
+
 #ifdef __WXWINCE__
     // Slower emulation since WinCE doesn't support Pie and Arc
-    double r = sqrt( (x1-xc)*(x1-xc) + (y1-yc)*(y1-yc) );
     double sa = acos((x1-xc)/r)/M_PI*180; // between 0 and 180
     if( y1>yc ) sa = -sa; // below center
     double ea = atan2(yc-y2, x2-xc)/M_PI*180;
@@ -689,11 +692,6 @@ void wxDC::DoDrawArc(wxCoord x1, wxCoord y1,
 
     wxColourChanger cc(*this); // needed for wxSTIPPLE_MASK_OPAQUE handling
 
-    double dx = xc - x1;
-    double dy = yc - y1;
-    double radius = (double)sqrt(dx*dx+dy*dy);
-    wxCoord r = (wxCoord)radius;
-
     // treat the special case of full circle separately
     if ( x1 == x2 && y1 == y2 )
     {
@@ -707,7 +705,9 @@ void wxDC::DoDrawArc(wxCoord x1, wxCoord y1,
     wxCoord yy2 = YLOG2DEV(y2);
     wxCoord xxc = XLOG2DEV(xc);
     wxCoord yyc = YLOG2DEV(yc);
-    wxCoord ray = (wxCoord) sqrt(double((xxc-xx1)*(xxc-xx1)+(yyc-yy1)*(yyc-yy1)));
+    dx = xxc - xx1;
+    dy = yyc - yy1;
+    wxCoord ray = (wxCoord)sqrt(dx*dx + dy*dy);
 
     wxCoord xxx1 = (wxCoord) (xxc-ray);
     wxCoord yyy1 = (wxCoord) (yyc-ray);
@@ -1123,6 +1123,17 @@ void wxDC::DoDrawEllipticArc(wxCoord x,wxCoord y,wxCoord w,wxCoord h,double sa,d
     rx2 += (int)(100.0 * abs(w) * cos(ea));
     ry2 -= (int)(100.0 * abs(h) * m_signY * sin(ea));
 
+    // Swap start and end positions if the end angle is less than the start angle.
+    if (ea < sa) {
+    int temp;
+    temp = rx2;
+    rx2 = rx1;
+    rx1 = temp;
+    temp = ry2;
+    ry2 = ry1;
+    ry1 = temp;
+    }
+
     // draw pie with NULL_PEN first and then outline otherwise a line is
     // drawn from the start and end points to the centre
     HPEN hpenOld = (HPEN) ::SelectObject(GetHdc(), (HPEN) ::GetStockObject(NULL_PEN));
@@ -1187,6 +1198,10 @@ void wxDC::DoDrawBitmap( const wxBitmap &bmp, wxCoord x, wxCoord y, bool useMask
             return;
     }
 
+#ifndef __WXWINCE__
+    StretchBltModeChanger changeMode(GetHdc(), COLORONCOLOR);
+#endif
+
     if ( useMask )
     {
         wxMask *mask = bmp.GetMask();
