@@ -511,8 +511,8 @@ class ImageDialog(wx.Dialog):
         if self.fl_ext == "All":
             all_files = []
 
-            if self.fl_types[-1:-1] == '*.*':
-                allTypes = self.fl_types[1:-1]
+            if self.fl_types[-1] == 'All Files':
+                allTypes = self.fl_types[-1:]
             else:
                 allTypes = self.fl_types[1:]
             for ftypes in allTypes:    # get list of all
