@@ -18,21 +18,23 @@ element is defined in the file.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cbrowser.py 63308 2010-01-30 00:03:47Z CJP $"
-__revision__ = "$Revision: 63308 $"
+__svnid__ = "$Id: cbrowser.py 67736 2011-05-13 22:42:50Z CJP $"
+__revision__ = "$Revision: 67736 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import StringIO
-import threading
 import wx
 
 # Editra Libraries
 import ed_glob
+import ebmlib
 from profiler import Profile_Get, Profile_Set
 import ed_msg
+import ed_thread
 
 # Local Imports
+import cbconfig
 import gentag.taglib as taglib
 from tagload import TagLoader
 import IconFile
@@ -43,6 +45,7 @@ ID_CODEBROWSER = wx.NewId()
 ID_BROWSER = wx.NewId()
 ID_GOTO_ELEMENT = wx.NewId()
 PANE_NAME = u"CodeBrowser"
+MSG_CODEBROWSER_MENU = ("CodeBrowser", "ContextMenu")
 _ = wx.GetTranslation
 
 # HACK for i18n scripts to pick up translation strings
@@ -53,7 +56,7 @@ STRINGS = ( _("Class Definitions"), _("Defines"), _("Function Definitions"),
             _("Sections"), _("Style Tags"), _("Subroutines"),
             _("Subroutine Declarations"), _("Task Definitions"), 
             _("Modules"), _("Functions"), _("Public Functions"),
-            _("Public Subroutines"), _("Imports") )
+            _("Public Subroutines"), _("Imports"), _("Elements") )
 del STRINGS
 
 #--------------------------------------------------------------------------#
@@ -62,12 +65,12 @@ class CodeBrowserTree(wx.TreeCtrl):
     def __init__(self, parent, id=ID_BROWSER,
                  pos=wx.DefaultPosition, size=wx.DefaultSize,
                  style=wx.TR_DEFAULT_STYLE|wx.TR_HIDE_ROOT):
-        wx.TreeCtrl.__init__(self, parent, id, pos, size, style)
+        super(CodeBrowserTree, self).__init__(parent, id, pos, size, style)
 
         # Attributes
         self._log = wx.GetApp().GetLog()
         self._mw = parent
-        self._menu = None
+        self._menu = ebmlib.ContextMenuManager()
         self._selected = None
         self._cjob = 0
         self._lastjob = u'' # Name of file in last sent out job
@@ -99,9 +102,11 @@ class CodeBrowserTree(wx.TreeCtrl):
         # Event Handlers
         self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnActivated)
         self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnContext)
+        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.OnExpanding)
         self.Bind(wx.EVT_MENU, self.OnMenu)
         self.Bind(wx.EVT_TIMER, self.OnStartJob, self._timer)
         self.Bind(wx.EVT_TIMER, lambda evt: self._SyncTree(), self._sync_timer)
+        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy)
         self.Bind(EVT_JOB_FINISHED, self.OnTagsReady)
 
         # Editra Message Handlers
@@ -111,6 +116,8 @@ class CodeBrowserTree(wx.TreeCtrl):
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_FILE_SAVED)
         ed_msg.Subscribe(self.OnSyncTree, ed_msg.EDMSG_UI_STC_POS_CHANGED)
         ed_msg.Subscribe(self.OnEditorRestore, ed_msg.EDMSG_UI_STC_RESTORE)
+        ed_msg.Subscribe(self.OnBrowserCfg,
+                         ed_msg.EDMSG_PROFILE_CHANGE + (cbconfig.CB_PROFILE_KEY,))
 
         # Backwards compatibility
         if hasattr(ed_msg, 'EDMSG_UI_STC_LEXER') and \
@@ -118,13 +125,15 @@ class CodeBrowserTree(wx.TreeCtrl):
             ed_msg.Subscribe(self.OnUpdateFont, ed_msg.EDMSG_DSP_FONT)
             ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_UI_STC_LEXER)
 
-    def __del__(self):
-        """Unsubscribe from messages on del"""
-        ed_msg.Unsubscribe(self.OnUpdateTree)
-        ed_msg.Unsubscribe(self.OnThemeChange)
-        ed_msg.Unsubscribe(self.OnUpdateFont)
-        ed_msg.Unsubscribe(self.OnSyncTree)
-        ed_msg.Unsubscribe(self.OnEditorRestore)
+    def OnDestroy(self, event):
+        """Unsubscribe from messages on destroy"""
+        if self:
+            self._menu.Clear()
+            ed_msg.Unsubscribe(self.OnUpdateTree)
+            ed_msg.Unsubscribe(self.OnThemeChange)
+            ed_msg.Unsubscribe(self.OnUpdateFont)
+            ed_msg.Unsubscribe(self.OnSyncTree)
+            ed_msg.Unsubscribe(self.OnEditorRestore)
 
     def _GetCurrentCtrl(self):
         """Get the current buffer"""
@@ -319,7 +328,7 @@ class CodeBrowserTree(wx.TreeCtrl):
         """
         # Check if there is a node for this Code object
         if self.nodes.get(obj.type, None) is None:
-            # Look for a description to use as catagory title
+            # Look for a description to use as category title
             if self._cdoc is not None:
                 desc = self._cdoc.GetElementDescription(obj.type).title()
             else:
@@ -334,7 +343,7 @@ class CodeBrowserTree(wx.TreeCtrl):
 
     def DeleteChildren(self, item):
         """Delete the children of a given node"""
-        wx.TreeCtrl.DeleteChildren(self, item)
+        super(CodeBrowserTree, self).DeleteChildren(item)
         for key in self.nodes.keys():
             self.nodes[key] = None
 
@@ -368,18 +377,20 @@ class CodeBrowserTree(wx.TreeCtrl):
 
     def OnContext(self, evt):
         """Show the context menu when an item is clicked on"""
-        if self._menu is not None:
-            self._menu.Destroy()
-            self._menu = None
-
+        self._menu.Clear()
+        self._menu.Menu = wx.Menu()
         tree_id = evt.GetItem()
         data = self.GetPyData(tree_id)
+        # Add the Goto menu option
         if data is not None:
+            self._menu.SetUserData("object", data)
             self._selected = tree_id # Store the selected
-            self._menu = wx.Menu()
             txt = self.GetItemText(self._selected).split('[')[0].strip()
-            self._menu.Append(ID_GOTO_ELEMENT, _("Goto \"%s\"") % txt)
-            self.PopupMenu(self._menu)
+            self._menu.Menu.Append(ID_GOTO_ELEMENT, _("Goto \"%s\"") % txt)
+        # Let clients hook into the context menu
+        # Call AddHandler to add a callback handler callback(menu_id, itemdata)
+        ed_msg.PostMessage(MSG_CODEBROWSER_MENU, self._menu)
+        self.PopupMenu(self._menu.Menu)
 
     def OnCompareItems(self, item1, item2):
         """Compare two tree items for sorting.
@@ -388,20 +399,49 @@ class CodeBrowserTree(wx.TreeCtrl):
         @return: -1, 0, 1
 
         """
-        txt1 = self.GetItemText(item1).lower()
-        txt2 = self.GetItemText(item2).lower()
-#        txt1 = self.GetPyData(item1)
-#        txt2 = self.GetPyData(item2)
-        if txt1 < txt2:
+        sortopt = cbconfig.GetSortOption()
+        if sortopt == cbconfig.CB_ALPHA_SORT:
+            # Sort items by name
+            val1 = self.GetItemText(item1).lower()
+            val2 = self.GetItemText(item2).lower()
+        else:
+            # Sort items by line number
+            val1 = self.GetPyData(item1)
+            val2 = self.GetPyData(item2)
+
+        if val1 < val2:
             return -1
-        elif txt1 == txt2:
+        elif val1 == val2:
             return 0
         else:
             return 1
 
+    def OnBrowserCfg(self, msg):
+        """Profile update message for when CodeBrowser
+        configuration is updated.
+
+        """
+        # Resort all items
+        def SortAllItems(parent):
+            item, cookie = self.GetFirstChild(parent)
+            while item:
+                if self.ItemHasChildren(item):
+                    if self.IsExpanded(item):
+                        self.SortChildren(item)
+                    SortAllItems(item)
+                item, cookie = self.GetNextChild(parent, cookie)
+        SortAllItems(self.root)
+
     def OnMenu(self, evt):
         """Handle the context menu events"""
-        if evt.GetId() == ID_GOTO_ELEMENT:
+        e_id  = evt.GetId()
+        handler = self._menu.GetHandler(e_id)
+        if handler:
+            data = None
+            if self._selected is not None:
+                data = self.GetPyData(self._selected)
+            handler(e_id, data)
+        elif e_id == ID_GOTO_ELEMENT:
             if self._selected is not None:
                 self.GotoElement(self._selected)
         else:
@@ -412,6 +452,11 @@ class CodeBrowserTree(wx.TreeCtrl):
         """Called when editor size is unmaximized"""
         self.OnUpdateTree()
 
+    def OnExpanding(self, evt):
+        """Update sorting"""
+        item = evt.GetItem()
+        self.SortChildren(item)
+
     def OnThemeChange(self, msg):
         """Update the images when Editra's theme changes
         @param msg: Message Object
@@ -500,9 +545,9 @@ class CodeBrowserTree(wx.TreeCtrl):
                                (self._mw.GetId(), -1, -1))
 
             # Create and start the worker thread
-            thread = TagGenThread(self, self._cjob, genfun,
-                                  StringIO.StringIO(self._cpage.GetText()))
-            wx.CallLater(75, thread.start)
+            task = TagGenJob(self, self._cjob, genfun,
+                             StringIO.StringIO(self._cpage.GetText()))
+            ed_thread.EdThreadPool().QueueJob(task.DoTask)
         else:
             self._ClearTree()
             ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW,
@@ -549,6 +594,14 @@ class CodeBrowserTree(wx.TreeCtrl):
             # Start the oneshot timer for beginning the tag generator job
             self._timer.Start(300, True)
     
+    def OnShowAUIPane(self):
+        """Interface method that the main Editra window will call
+        When its auimanager does a Show when this window is contained
+        within an aui pane. Forces the tree to update.
+
+        """
+        self.OnUpdateTree(force=True)
+
     def OnShowBrowser(self, evt):
         """Show the browser pane
         @param evt: wx.MenuEvent
@@ -559,7 +612,7 @@ class CodeBrowserTree(wx.TreeCtrl):
             pane = mgr.GetPane(PANE_NAME)
             pane.Show(not pane.IsShown())
             mgr.Update()
-            self.OnUpdateTree(force=True)
+            self.OnShowAUIPane()
         else:
             evt.Skip()
 
@@ -596,8 +649,8 @@ class CodeBrowserTree(wx.TreeCtrl):
         self._SyncTree()
 
 #--------------------------------------------------------------------------#
-# Tag Generator Thread
-class TagGenThread(threading.Thread):
+# Tag Generator
+class TagGenJob(object):
     """Thread for running tag parser on and returning the results for
     display in the tree.
 
@@ -610,7 +663,7 @@ class TagGenThread(threading.Thread):
         @param buff: string buffer to pass to genfun
 
         """
-        threading.Thread.__init__(self)
+        super(TagGenJob, self).__init__()
 
         # Attributes
         self.reciever = reciever
@@ -618,7 +671,7 @@ class TagGenThread(threading.Thread):
         self.buff = buff
         self.task = genfun
 
-    def run(self):
+    def DoTask(self):
         """Run the generator function and return the docstruct to
         the main thread.
 
