@@ -16,11 +16,11 @@ configuration and presentation options.
 The following items are the options that the AdvancedFindReplaceDialog offers
 over the basic FindReplaceDialog.
 
-  * Hide/Show each option or section indvidually (basic dialog only disables them)
+  * Hide/Show each option or section individually (basic dialog only disables them)
   * Multi-Find/Replace event action for Find All / Replace All actions
   * Switch dialog from Find mode to Replace mode or visa-versa once its already
     been created.
-  * Options for specifiying the location to look in
+  * Options for specifying the location to look in.
   * Regular Expression option
   * Use standard dialog or a floating MiniFrame (default)
 
@@ -35,8 +35,30 @@ eclib.ctrlbox
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: finddlg.py 55493 2008-09-06 02:18:56Z CJP $"
-__revision__ = "$Revision: 55493 $"
+__svnid__ = "$Id: finddlg.py 68232 2011-07-12 02:08:53Z CJP $"
+__revision__ = "$Revision: 68232 $"
+
+__all__ = ["FindBox", "FindEvent", "FindPanel", "FindReplaceDlg",
+           "MiniFindReplaceDlg", "AdvFindReplaceDlg",
+
+           "AFR_STYLE_FINDDIALOG", "AFR_STYLE_REPLACEDIALOG",
+           "AFR_STYLE_NON_FLOATING", "AFR_STYLE_NO_MODE_SELECT",
+           
+           "AFR_UP", "AFR_WHOLEWORD", 
+           "AFR_MATCHCASE", "AFR_REGEX", "AFR_RECURSIVE", "AFR_NOLOOKIN",
+           "AFR_NOUPDOWN", "AFR_NOWHOLEWORD", "AFR_NOMATCHCASE", "AFR_NOREGEX",
+           "AFR_NOFILTER", "AFR_NOOPTIONS", "AFR_NO_COUNT", "AFR_NO_ALL_BTN",
+           "AFR_SIMPLE",
+
+           "LOCATION_CURRENT_DOC", "LOCATION_IN_SELECTION",
+           "LOCATION_OPEN_DOCS", "LOCATION_IN_CURRENT_DIR",
+           "LOCATION_IN_FILES", "LOCATION_MAX",
+
+           "edEVT_FIND_CLOSE", "EVT_FIND_CLOSE", "edEVT_FIND", "EVT_FIND",
+           "edEVT_FIND_NEXT", "EVT_FIND_NEXT", "edEVT_FIND_ALL", "EVT_FIND_ALL",
+           "edEVT_REPLACE", "EVT_REPLACE", "edEVT_REPLACE_ALL",
+           "EVT_REPLACE_ALL", "EVT_OPTION_CHANGED", "edEVT_OPTION_CHANGED",
+           "EVT_COUNT", "edEVT_COUNT"]
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -51,27 +73,38 @@ import platebtn
 # Globals
 
 # Style Flags
-AFR_STYLE_FINDDIALOG    = 0
-AFR_STYLE_REPLACEDIALOG = 1
-AFR_STYLE_NON_FLOATING  = 2
+AFR_STYLE_FINDDIALOG        = 0 # Start dialog in Find mode
+AFR_STYLE_REPLACEDIALOG     = 1 # Start dialog in Replace mode
+AFR_STYLE_NON_FLOATING      = 2 # Use a standard dialog as the tlw
+AFR_STYLE_NO_MODE_SELECT    = 4 # Hide the mode selector buttons
 
 # FindReplaceData Flags
-AFR_UP          = 1
-AFR_WHOLEWORD   = 2
-AFR_MATCHCASE   = 4
-AFR_REGEX       = 8
-AFR_RECURSIVE   = 16
-AFR_NOLOOKIN    = 32
-AFR_NOUPDOWN    = 64
-AFR_NOWHOLEWORD = 128
-AFR_NOMATCHCASE = 256
-AFR_NOREGEX     = 512
-AFR_NOOPTIONS   = 1024
+AFR_UP          = 1             # Set dialogs Search Up flag
+AFR_WHOLEWORD   = 2             # Set dialogs whole word search flag
+AFR_MATCHCASE   = 4             # Set dialogs match case search flag
+AFR_REGEX       = 8             # Set dialogs use regular expression flag
+AFR_RECURSIVE   = 16            # Set dialogs recursive directory search flag
+AFR_NOLOOKIN    = 32            # Hide the Looking combobox in the dialog
+AFR_NOUPDOWN    = 64            # Hide the Direction options in the dialog
+AFR_NOWHOLEWORD = 128           # Hide the Whole Word option in the dialog
+AFR_NOMATCHCASE = 256           # Hide the Match Case option in the dialog
+AFR_NOREGEX     = 512           # Hide the Regular Expression option
+AFR_NOFILTER    = 1024          # Hide the File Filter option
+AFR_NOOPTIONS   = 2048          # Hide all options in the dialog
+AFR_NO_COUNT    = 4096          # Hide Count Button
+AFR_NO_ALL_BTN  = 8192          # Hide Find All / Replace All buttons
+
+# Convenience Flags
+AFR_SIMPLE = (AFR_NOLOOKIN | AFR_NOOPTIONS | AFR_NOUPDOWN | \
+              AFR_NO_COUNT | AFR_NO_ALL_BTN)
 
 # Search Location Parameters (NOTE: must be kept in sync with Lookin List)
-LOCATION_CURRENT_DOC = 0
-LOCATION_OPEN_DOCS   = 1
-LOCATION_IN_FILES    = 2
+LOCATION_CURRENT_DOC    = 0
+LOCATION_IN_SELECTION   = 1
+LOCATION_OPEN_DOCS      = 2
+LOCATION_IN_CURRENT_DIR = 3
+LOCATION_IN_FILES       = 4
+LOCATION_MAX            = LOCATION_IN_CURRENT_DIR
 
 # Control Names
 FindBoxName = "EdFindBox"
@@ -85,37 +118,58 @@ ID_MATCH_CASE = wx.NewId()
 ID_WHOLE_WORD = wx.NewId()
 ID_REGEX = wx.NewId()
 ID_RECURSE = wx.NewId()
+ID_COUNT = wx.NewId()
 ID_FIND_ALL = wx.NewId()
 ID_REPLACE_ALL = wx.NewId()
+ID_OPTION_CHANGE = wx.NewId()
 ID_CHOOSE_DIR = wx.NewId()
+ID_DIR_BOX = wx.NewId()
 
 _ = wx.GetTranslation
 
 #--------------------------------------------------------------------------#
 
-# Events
+# Events Definitions
+
+# Find dialog has been closed
 edEVT_FIND_CLOSE = wx.NewEventType()
 EVT_FIND_CLOSE = wx.PyEventBinder(edEVT_FIND_CLOSE, 1)
 
+# Find cutton clicked
 edEVT_FIND = wx.NewEventType()
 EVT_FIND = wx.PyEventBinder(edEVT_FIND, 1)
 
+# Find cutton clicked again with the same search string
 edEVT_FIND_NEXT = wx.NewEventType()
 EVT_FIND_NEXT = wx.PyEventBinder(edEVT_FIND_NEXT, 1)
 
+# Find All button clicked
 edEVT_FIND_ALL = wx.NewEventType()
 EVT_FIND_ALL = wx.PyEventBinder(edEVT_FIND_ALL, 1)
 
+# Replace button clicked
 edEVT_REPLACE = wx.NewEventType()
 EVT_REPLACE = wx.PyEventBinder(edEVT_REPLACE, 1)
 
+# Replace button clicked
+edEVT_COUNT = wx.NewEventType()
+EVT_COUNT = wx.PyEventBinder(edEVT_COUNT, 1)
+
+# Replace All button clicked
 edEVT_REPLACE_ALL = wx.NewEventType()
 EVT_REPLACE_ALL = wx.PyEventBinder(edEVT_REPLACE_ALL, 1)
 
-EVENT_MAP = { wx.ID_FIND : edEVT_FIND,
-              wx.ID_REPLACE : edEVT_REPLACE,
-              ID_FIND_ALL : edEVT_FIND_ALL,
-              ID_REPLACE_ALL : edEVT_REPLACE_ALL }
+# Find option has changed
+edEVT_OPTION_CHANGED = wx.NewEventType()
+EVT_OPTION_CHANGED = wx.PyEventBinder(edEVT_OPTION_CHANGED, 1)
+
+# Convenience for generating events
+_EVENT_MAP = { wx.ID_FIND : edEVT_FIND,
+               wx.ID_REPLACE : edEVT_REPLACE,
+               ID_COUNT : edEVT_COUNT,
+               ID_FIND_ALL : edEVT_FIND_ALL,
+               ID_REPLACE_ALL : edEVT_REPLACE_ALL,
+               ID_OPTION_CHANGE : edEVT_OPTION_CHANGED }
 
 class FindEvent(wx.PyCommandEvent):
     """Event sent by the FindReplaceDialog that contains all
@@ -128,7 +182,7 @@ class FindEvent(wx.PyCommandEvent):
         @keyword flags: Find/Replace flags
 
         """
-        wx.PyCommandEvent.__init__(self, etype, eid)
+        super(FindEvent, self).__init__(etype, eid)
 
         # Attributes
         self._flags = flags
@@ -136,6 +190,15 @@ class FindEvent(wx.PyCommandEvent):
         self._find = u''
         self._replace = u''
         self._dir = u''
+        self._filters = None
+        self._count = 0
+
+    def GetCount(self):
+        """Get the number of matches
+        @return: int
+
+        """
+        return self._count
 
     def GetDirectory(self):
         """Get the directory of files to search in
@@ -144,6 +207,13 @@ class FindEvent(wx.PyCommandEvent):
         """
         return self._dir
 
+    def GetFileFilters(self):
+        """Get the filter to search with
+        @return: list or None
+
+        """
+        return self._filters
+
     def GetFindString(self):
         """Get the find string
         @return: string
@@ -172,6 +242,13 @@ class FindEvent(wx.PyCommandEvent):
         """
         return self._loc
 
+    def SetCount(self, count):
+        """Set the count
+        @param count: int
+
+        """
+        self._count = count
+
     def SetDirectory(self, directory):
         """Set the directory of files to search in
         @param directory: string
@@ -179,6 +256,19 @@ class FindEvent(wx.PyCommandEvent):
         """
         self._dir = directory
 
+    def SetFileFilters(self, filters):
+        """Set the file filters to use for a Search In Files event
+        @param filters: string or list of strings
+
+        """
+        if isinstance(filters, basestring):
+            self._filters = filters.split()
+        else:
+            self._filters = filters
+
+        if self._filters is not None:
+            self._filters = [ f.strip() for f in self._filters ]
+
     def SetFindString(self, find):
         """Set the find String
         @param find: string
@@ -259,12 +349,14 @@ def AdvFindReplaceDlg(parent, fdata, title, style=AFR_STYLE_FINDDIALOG):
         dlg = FindReplaceDlg(parent, fdata, title, style)
     else:
         dlg = MiniFindReplaceDlg(parent, fdata, title, style)
+
     return dlg
 
 #--------------------------------------------------------------------------#
 
 class FindReplaceDlgBase:
-    """Mixin Base class for deriving FindReplaceDialogs.
+    """Delegate mixin base class for deriving FindReplaceDialogs, delegates
+    calls to the widgets and containers that make up the dialog.
     @note: The mixin must be initialized after the class its being mixed into
 
     """
@@ -294,9 +386,11 @@ class FindReplaceDlgBase:
 
         # Event handlers
         self.Bind(_EVT_MODE_CHANGE, self._OnModeChange)
+        self.Bind(_EVT_DO_CLOSE_DLG, lambda evt: self._SendCloseEvent())
         self.Bind(wx.EVT_MENU, lambda evt: self._SendCloseEvent(), id=wx.ID_CLOSE)
         self.Bind(wx.EVT_SET_FOCUS,
                   lambda evt: self._panel.SetFocus() and evt.Skip())
+        self.Bind(wx.EVT_SHOW, lambda evt: self._panel.SetFocus())
 
     def __DoLayout(self):
         """Layout the dialog"""
@@ -312,8 +406,8 @@ class FindReplaceDlgBase:
         data.SetFindString(self._panel._ftxt.GetValue())
         data.SetReplaceString(self._panel._rtxt.GetValue())
         evt = FindEvent(edEVT_FIND_CLOSE, self.GetId())
-        wx.PostEvent(self.GetParent(), evt)
         self.Hide()
+        wx.PostEvent(self.GetParent(), evt)
 
     def _OnModeChange(self, evt):
         """Update the the dialog when the mode changes"""
@@ -334,6 +428,27 @@ class FindReplaceDlgBase:
         """
         return self._panel.GetPanelMode()
 
+    def GetFileFilters(self):
+        """Get the file filters field value
+        @return: string
+
+        """
+        return self._panel.GetFileFilters()
+
+    def GetLookinChoices(self):
+        """Get the set choices from the looking choice list
+        @return: list of strings
+
+        """
+        return self._panel.GetLookinChoices()
+
+    def GetLookinSelection(self):
+        """Get the index of the selected item in the lookin choice list
+        @return: int
+
+        """
+        return self._panel.GetLookinSelection()
+
     def RefreshFindReplaceFields(self):
         """Refresh the values of the Find and Replace fields with the
         values that are currently in the FindReplaceData.
@@ -343,6 +458,13 @@ class FindReplaceDlgBase:
         self._panel.SetFindString(data.GetFindString())
         self._panel.SetReplaceString(data.GetReplaceString())
 
+    def RefreshFindOptions(self):
+        """Refresh the find options controls based on the current
+        values of the FindData owned by this window.
+
+        """
+        self._panel.RefreshControls()
+
     def SetData(self, data):
         """Set the dialogs FindReplaceData
         @param data: FindReplaceData
@@ -358,6 +480,13 @@ class FindReplaceDlgBase:
         """
         self._box.SetBoxMode(mode == AFR_STYLE_FINDDIALOG)
 
+    def SetFileFilters(self, filters):
+        """Set the file filters field value
+        @param filters: string
+
+        """
+        self._panel.SetFileFilters(filters)
+
     def SetFindBitmap(self, bmp):
         """Set the find Bitmap
         @param bmp: wx.Bitmap
@@ -365,7 +494,7 @@ class FindReplaceDlgBase:
         """
         self._box.SetFindBitmap(bmp)
 
-    def SetFlag(self, flag):
+    def SetFlag(self, flags):
         """Set a search dialog flag.
         @param flags: AFR_*
 
@@ -379,6 +508,29 @@ class FindReplaceDlgBase:
         """
         self._panel.SetFlags(flags)
 
+    def SetLookinChoices(self, paths):
+        """Set the looking choices
+        @param paths: list of strings
+
+        """
+        self._panel.SetLookinChoices(paths)
+
+    def SetLookinPath(self, path):
+        """Set the lookin path, adding it to the collection if it is
+        not in there.
+        @param path: string (path of directory)
+
+        """
+        idx = self._panel.AddLookinPath(path)
+        self.SetLookinSelection(idx)
+
+    def SetLookinSelection(self, sel):
+        """Set the looking choices selection
+        @param sel: int
+
+        """
+        self._panel.SetLookinSelection(sel)
+
     def SetReplaceBitmap(self, bmp):
         """Set the replace bitmap
         @param bmp: wx.Bitmap
@@ -402,7 +554,7 @@ class FindReplaceDlgBase:
 
     def SetReplaceString(self, rstring):
         """Set the replace controls search string
-        @param query: string
+        @param rstring: string
 
         """
         self._panel.SetReplaceString(rstring)
@@ -470,8 +622,8 @@ class FindBox(ctrlbox.ControlBox):
         @param fdata: wx.FindReplaceData
 
         """
-        ctrlbox.ControlBox.__init__(self, parent, id, pos, size,
-                                    wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
+        super(FindBox, self).__init__(parent, id, pos, size,
+                                      wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
 
         # Attributes
         self._fpanel = FindPanel(self, fdata, style=style)
@@ -486,13 +638,17 @@ class FindBox(ctrlbox.ControlBox):
 
         # Setup
         if wx.Platform == '__WXGTK__':
-            ctrlbar.SetWindowStyle(ctrlbox.CTRLBAR_STYLE_DEFAULT)
+            ctrlbar.SetWindowStyle(ctrlbox.CTRLBAR_STYLE_BORDER_BOTTOM)
+
         ctrlbar.SetVMargin(2, 2)
         ctrlbar.AddControl(self.find, wx.ALIGN_LEFT)
         ctrlbar.AddControl(self.replace, wx.ALIGN_LEFT)
         self.SetControlBar(ctrlbar)
         self.SetWindow(self._fpanel)
 
+        if style & AFR_STYLE_NO_MODE_SELECT:
+            self.GetControlBar().Hide()
+
         # Event Handlers
         self.Bind(wx.EVT_BUTTON, self.OnButton)
 
@@ -522,7 +678,7 @@ class FindBox(ctrlbox.ControlBox):
         """
         self._fpanel.SetFindMode(find)
         self.Layout()
-        evt = _DlgModeChange(self.GetId(), _edEVT_MODE_CHANGE)
+        evt = _AdvFindDlgInternalEvent(self.GetId(), _edEVT_MODE_CHANGE)
         wx.PostEvent(self.GetParent(), evt)
 
     def SetReplaceBitmap(self, bmp):
@@ -537,15 +693,15 @@ class FindBox(ctrlbox.ControlBox):
 
 class FindPanel(wx.Panel):
     """Find controls panel"""
-    def __init__(self, parent, fdata, id=wx.ID_ANY, pos=wx.DefaultPosition,
-                 size=wx.DefaultSize, style=AFR_STYLE_FINDDIALOG,
-                 name=FindPanelName):
+    def __init__(self, parent, fdata, id=wx.ID_ANY, 
+                 pos=wx.DefaultPosition, size=wx.DefaultSize,
+                 style=AFR_STYLE_FINDDIALOG, name=FindPanelName):
         """Create the panel
         @param fdata: wx.FindReplaceData
 
         """
-        wx.Panel.__init__(self, parent, id, pos, size,
-                          wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
+        super(FindPanel, self).__init__(parent, id, pos, size,
+                                        wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
 
         # Attributes
         # TODO: change to editable combo box when wxMac has native widget
@@ -553,9 +709,12 @@ class FindPanel(wx.Panel):
         self._mode = style
         self._ftxt = wx.TextCtrl(self, value=fdata.GetFindString())
         self._rtxt = wx.TextCtrl(self, value=fdata.GetReplaceString())
-        locations = [_("Current Document"), _("Open Documents")]
+        locations = [_("Current Document"), _("Selected Text"),
+                     _("Open Documents"), _("Current Directory")]
         self._lookin = wx.Choice(self, ID_LOOKIN, choices=locations)
         self._lookin.SetSelection(0)
+        self._filterlbl = wx.StaticText(self, label=_("File Filters:"))
+        self._filters = None # Created in __DoLayout
         self._sizers = dict()
         self._paths = dict()
         self._fdata = fdata
@@ -575,10 +734,23 @@ class FindPanel(wx.Panel):
                   lambda evt: self.FireEvent(evt.GetId()) or evt.Skip())
         self.Bind(wx.EVT_CHECKBOX, self.OnOption)
         self.Bind(wx.EVT_RADIOBUTTON, self.OnOption)
-        self.Bind(wx.EVT_CHOICE, lambda evt: self._UpdateContext(), id=ID_LOOKIN)
-        for bid in (wx.ID_FIND, wx.ID_REPLACE, ID_FIND_ALL, ID_REPLACE_ALL):
+        self.Bind(wx.EVT_CHOICE, self.OnChoice, id=ID_LOOKIN)
+        for bid in (wx.ID_FIND, wx.ID_REPLACE, ID_COUNT,
+                    ID_FIND_ALL, ID_REPLACE_ALL):
             self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUI, id=bid)
+        self.Bind(wx.EVT_SET_FOCUS, lambda evt: self.__SetFocus())
         self._ftxt.Bind(wx.EVT_SET_FOCUS, lambda evt: self._ftxt.SelectAll())
+        self._rtxt.Bind(wx.EVT_SET_FOCUS, lambda evt: self._rtxt.SelectAll())
+
+        # Key handling HACK for windows
+        if wx.Platform == '__WXMSW__':
+            for child in self.GetChildren():
+                child.Bind(wx.EVT_KEY_UP, self._OnKeyUp)
+
+    def __SetFocus(self):
+        """Set the focus to the find box"""
+        self._ftxt.SetFocus()
+        self._ftxt.SelectAll()
 
     def __DoLayout(self):
         """Layout the panel"""
@@ -615,6 +787,7 @@ class FindPanel(wx.Panel):
         self._sizers['look'] = wx.BoxSizer(wx.VERTICAL)
         li_sz = wx.BoxSizer(wx.HORIZONTAL)
         dirbtn = wx.Button(self, ID_CHOOSE_DIR, u"...", style=wx.BU_EXACTFIT)
+        dirbtn.SetToolTipString(_("Choose Folder"))
         li_sz.AddMany([(self._lookin, 1, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
                        (dirbtn, 0, wx.ALIGN_CENTER_VERTICAL)])
         li_lbl = wx.StaticText(self, label=_("Look in") + u":")
@@ -626,7 +799,7 @@ class FindPanel(wx.Panel):
 
         # Search Direction Box
         self._sizers['dir'] = wx.BoxSizer(wx.VERTICAL)
-        dbox = wx.StaticBox(self, label=_("Direction"))
+        dbox = wx.StaticBox(self, id=ID_DIR_BOX, label=_("Direction"))
         dboxsz = wx.StaticBoxSizer(dbox, wx.HORIZONTAL)
         dboxsz.AddMany([(wx.RadioButton(self, wx.ID_UP, _("Up")), 0),
                         ((20, 5), 0),
@@ -643,6 +816,24 @@ class FindPanel(wx.Panel):
                           (ID_REGEX, _("Regular expression")),
                           (ID_RECURSE, _("Search Recursively"))]:
             sboxsz.AddMany([((3, 3), 0), (wx.CheckBox(self, cid, clbl), 0)])
+
+        # File Filters
+        self._sizers['filter'] = wx.BoxSizer(wx.VERTICAL)
+        self._filters = wx.TextCtrl(self)
+        tt_txt = _("Enter wildcard shell patterns for matching files (*.txt).")
+        self._filters.SetToolTipString(tt_txt)
+
+        # Disable spell checking on mac for this control
+        if wx.Platform == '__WXMAC__':
+            self._filters.MacCheckSpelling(False)
+
+        f_sz = wx.BoxSizer(wx.HORIZONTAL)
+        f_sz.Add(self._filters, 1, wx.EXPAND)
+        self._sizers['filter'].AddMany([(self._filterlbl, 0, wx.ALIGN_LEFT),
+                                        ((3, 3), 0), (f_sz, 0, wx.EXPAND),
+                                        ((5, 5), 0)])
+        sboxsz.AddMany([((3, 3), 0), (self._sizers['filter'], 0, wx.EXPAND)])
+
         self._sizers['opt'].AddMany([((5, 5), 0), (sboxsz, 0, wx.EXPAND)])
 
         # Buttons
@@ -651,6 +842,7 @@ class FindPanel(wx.Panel):
         self._sizers['frspacer'] = bsizer.Add((50, 1), 1)
         for bid, blbl in [(wx.ID_FIND, _("Find")),
                           (wx.ID_REPLACE, _("Replace")),
+                          (ID_COUNT, _("Count")),
                           (ID_FIND_ALL, _("Find All")),
                           (ID_REPLACE_ALL, _("Replace All"))]:
             self._sizers[bid] = wx.BoxSizer(wx.HORIZONTAL)
@@ -683,10 +875,21 @@ class FindPanel(wx.Panel):
         self.FindWindowById(ID_WHOLE_WORD).Enable(not (flags & AFR_NOWHOLEWORD))
         self.FindWindowById(ID_MATCH_CASE).Enable(not (flags & AFR_NOMATCHCASE))
         self.FindWindowById(ID_REGEX).Enable(not (flags & AFR_NOREGEX))
-        in_files = bool(self._lookin.GetSelection() >= LOCATION_IN_FILES)
+        in_files = bool(self._lookin.GetSelection() >= LOCATION_MAX)
         recurse = self.FindWindowById(ID_RECURSE)
         recurse.SetValue(flags & AFR_RECURSIVE)
         recurse.Enable(in_files)
+        self.ShowFileFilters(in_files and not (flags & AFR_NOFILTER))
+
+    def _OnKeyUp(self, evt):
+        """Check if the dialog should be closed
+        @note: msw only
+
+        """
+        if evt.GetKeyCode() == wx.WXK_ESCAPE:
+            evt = _AdvFindDlgInternalEvent(self.GetId(), _edEVT_DO_CLOSE_DLG)
+            wx.PostEvent(self.GetTopLevelParent(), evt)
+        evt.Skip()
 
     def _ShowButtons(self, find=True):
         """Toggle the visiblity of a button set
@@ -694,11 +897,25 @@ class FindPanel(wx.Panel):
 
         """
         if find:
-            show = (wx.ID_FIND, ID_FIND_ALL, 'fspacer')
-            hide = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
+            show = [wx.ID_FIND, ID_COUNT, ID_FIND_ALL, 'fspacer']
+            hide = [wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer']
         else:
-            show = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
-            hide = (ID_FIND_ALL, 'fspacer')
+            show = [wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer']
+            hide = [ID_FIND_ALL, ID_COUNT, 'fspacer']
+
+        # Hide extra buttons as per configured preference
+        flags = self._fdata.GetFlags()
+        if flags & AFR_NO_COUNT and ID_COUNT in show:
+            show.remove(ID_COUNT)
+            hide.append(ID_COUNT)
+
+        if flags & AFR_NO_ALL_BTN:
+            if ID_FIND_ALL in show:
+                show.remove(ID_FIND_ALL)
+                hide.append(ID_FIND_ALL)
+            if ID_REPLACE_ALL in show:
+                show.remove(ID_REPLACE_ALL)
+                hide.append(ID_REPLACE_ALL)
 
         for ctrl in show:
             if isinstance(ctrl, basestring):
@@ -717,8 +934,18 @@ class FindPanel(wx.Panel):
 
         """
         self._UpdateDefaultBtn()
-        in_files = bool(self._lookin.GetSelection() >= LOCATION_IN_FILES)
+        in_files = bool(self._lookin.GetSelection() >= LOCATION_MAX)
         self.FindWindowById(ID_RECURSE).Enable(in_files)
+        flags = self._fdata.GetFlags()
+
+        # Disable direction settings when searcing in file since they
+        # are ignored anyway.
+        for cid in (wx.ID_UP, wx.ID_DOWN, ID_DIR_BOX):
+            self.FindWindowById(cid).Enable(not in_files)
+
+        # Only update visibility of file filter field if it is enabled
+        if not (flags & AFR_NOFILTER):
+            self.ShowFileFilters(in_files)
 
     def _UpdateDefaultBtn(self):
         """Change the default button depending on what the search context
@@ -729,18 +956,28 @@ class FindPanel(wx.Panel):
         find_all = self.FindWindowById(ID_FIND_ALL)
         replace = self.FindWindowById(wx.ID_REPLACE)
         replace_all = self.FindWindowById(ID_REPLACE_ALL)
+        count = self.FindWindowById(ID_COUNT)
 
-        if self._lookin.GetSelection() > LOCATION_CURRENT_DOC:
+        lookin = self._lookin.GetSelection()
+        if  lookin > LOCATION_IN_SELECTION:
             if self._mode == AFR_STYLE_FINDDIALOG:
                 find_all.SetDefault()
             else:
                 replace_all.SetDefault()
             find.Disable()
             replace.Disable()
+            count.Disable()
+        elif lookin == LOCATION_IN_SELECTION:
+            find.Disable()
+            replace.Disable()
+            replace_all.SetDefault()
         else:
             find.SetDefault()
             find.Enable()
             replace.Enable()
+            count.Enable()
+
+    #------------------------------------------------------#
 
     def AddLookinPath(self, path):
         """Add a path to the lookin path collection
@@ -776,7 +1013,7 @@ class FindPanel(wx.Panel):
 
         """
         flags = self._fdata.GetFlags()
-        flags &= ~flag 
+        flags &= ~flag
         self.SetFlags(flags)
 
     def FireEvent(self, eid):
@@ -784,7 +1021,7 @@ class FindPanel(wx.Panel):
         @param eid: Event id
 
         """
-        etype = EVENT_MAP.get(eid, None)
+        etype = _EVENT_MAP.get(eid, None)
         query = self._ftxt.GetValue()
         if eid == wx.ID_FIND:
             if self._lastSearch == query:
@@ -809,6 +1046,7 @@ class FindPanel(wx.Panel):
 
             if stype >= LOCATION_IN_FILES:
                 evt.SetDirectory(self._paths.get(lookin_idx, u''))
+                evt.SetFileFilters(self._filters.GetValue())
 
             wx.PostEvent(self.GetParent(), evt)
             return True
@@ -822,6 +1060,30 @@ class FindPanel(wx.Panel):
         """
         return self._fdata
 
+    def GetFileFilters(self):
+        """Get the currently set file filters
+        @return: string
+
+        """
+        return self._filters.GetValue()
+
+    def GetLookinChoices(self):
+        """Get the looking choices
+        @return: list of strings
+
+        """
+        choices = list()
+        for key in sorted(self._paths.keys()):
+            choices.append(self._paths[key])
+        return choices
+
+    def GetLookinSelection(self):
+        """Get the index of the currently selected lookin choice
+        @return: int
+
+        """
+        return self._lookin.GetSelection()
+
     def GetPanelMode(self):
         """Get the current display mode of the panel
         @return: AFR_STYLE_FINDDIALOG or AFR_STYLE_REPLACEDIALOG
@@ -829,6 +1091,23 @@ class FindPanel(wx.Panel):
         """
         return self._mode
 
+    def OnChoice(self, evt):
+        """Handle choice control events
+        @param evt: wx.EVT_CHOICE
+
+        """
+        e_id = evt.GetId()
+        if e_id == ID_LOOKIN:
+            self._UpdateContext()
+            choice = self._lookin.GetSelection()
+            if choice >= LOCATION_IN_FILES:
+                tts = self._paths.get(choice, u'')
+                self._lookin.SetToolTipString(tts)
+            else:
+                self._lookin.SetToolTipString(u'')
+        else:
+            evt.Skip()
+
     def OnChooseDir(self, evt):
         """Open the choose directory dialog for selecting what
         path to do a search in files in.
@@ -840,7 +1119,8 @@ class FindPanel(wx.Panel):
             if dlg.ShowModal() == wx.ID_OK:
                 path = dlg.GetPath()
                 if path is not None and len(path):
-                    self.SetLookinSelection(path)
+                    idx = self.AddLookinPath(path)
+                    self.SetLookinSelection(idx)
                 self._UpdateContext()
             dlg.Destroy()
         else:
@@ -863,8 +1143,10 @@ class FindPanel(wx.Panel):
                 self.SetFlag(fmap[eid])
             else:
                 self.ClearFlag(fmap[eid])
+            self.FireEvent(ID_OPTION_CHANGE)
         elif eid == wx.ID_DOWN:
             self.ClearFlag(fmap[wx.ID_UP])
+            self.FireEvent(ID_OPTION_CHANGE)
         else:
             evt.Skip()
 
@@ -873,7 +1155,8 @@ class FindPanel(wx.Panel):
         @param evt: wx.UpdateUIEvent
 
         """
-        if evt.GetId() in (wx.ID_FIND, wx.ID_REPLACE) and \
+        # TODO: support count in files?
+        if evt.GetId() in (wx.ID_FIND, ID_COUNT, wx.ID_REPLACE) and \
            self._lookin.GetSelection() > LOCATION_CURRENT_DOC:
             evt.Enable(False)
         else:
@@ -904,6 +1187,10 @@ class FindPanel(wx.Panel):
         self._ftxt.SetValue(query)
         self._fdata.SetFindString(query)
 
+    def RefreshControls(self):
+        """Refresh the state of the controls from the current FindData."""
+        self._ConfigureControls()
+
     def SetData(self, data):
         """Set the FindReplaceData and update the dialog with that data
         @param data: wxFindReplaceData
@@ -914,6 +1201,13 @@ class FindPanel(wx.Panel):
         self._fdata = data
         self._ConfigureControls()
 
+    def SetFileFilters(self, filters):
+        """Set the file filters field values
+        @param filters: string
+
+        """
+        self._filters.SetValue(filters)
+
     def SetFlag(self, flag):
         """Set a search flag
         @param flag: AFR_* flag value
@@ -931,16 +1225,28 @@ class FindPanel(wx.Panel):
         self._fdata.SetFlags(flags)
         self._ConfigureControls()
 
-    def SetLookinSelection(self, path):
-        """Set the selection of the lookin control. If the given
-        path is not already stored it will be added.
-        @param path: string
+    def SetLookinChoices(self, paths):
+        """Set the looking choices
+        @param paths: list of strings
 
         """
-        idx = self.AddLookinPath(path)
-        if idx is not None:
+        for path in paths:
+            self.AddLookinPath(path)
+
+    def SetLookinSelection(self, idx):
+        """Set the selection of the lookin control.
+        @param idx: int
+
+        """
+        if idx <= self._lookin.GetCount():
             self._lookin.SetSelection(idx)
 
+            # If the selection is a Look in Files make sure the filter
+            # field is shown
+            flags = self._fdata.GetFlags()
+            in_files = bool(idx >= LOCATION_MAX)
+            self.ShowFileFilters(in_files and not (flags & AFR_NOFILTER))
+
     def SetReplaceString(self, rstring):
         """Set the replace fields string
         @param rstring: string
@@ -958,6 +1264,19 @@ class FindPanel(wx.Panel):
             self._sizers['dir'].ShowItems(show)
             self.Layout()
 
+    def ShowFileFilters(self, show=True):
+        """Show or hide the File Filters filed
+        @keyword show: bool
+
+        """
+        if 'filter' in self._sizers:
+            self._sizers['filter'].ShowItems(show)
+            self.Layout()
+
+            # Require additional resizing
+            evt = _AdvFindDlgInternalEvent(self.GetId(), _edEVT_MODE_CHANGE)
+            wx.PostEvent(self.GetTopLevelParent(), evt)
+
     def ShowLookinCombo(self, show=True):
         """Show the lookin choice and directory chooser control
         @keyword show: bool
@@ -982,7 +1301,10 @@ class FindPanel(wx.Panel):
 _edEVT_MODE_CHANGE = wx.NewEventType()
 _EVT_MODE_CHANGE = wx.PyEventBinder(_edEVT_MODE_CHANGE, 1)
 
-class _DlgModeChange(wx.PyEvent):
+_edEVT_DO_CLOSE_DLG = wx.NewEventType()
+_EVT_DO_CLOSE_DLG = wx.PyEventBinder(_edEVT_DO_CLOSE_DLG, 1)
+
+class _AdvFindDlgInternalEvent(wx.PyEvent):
     def __init__(self, winid=wx.ID_ANY, etype=wx.wxEVT_NULL):
         """Create the Event"""
         wx.PyEvent.__init__(self, winid, etype)
