@@ -14,7 +14,7 @@ import params
 Component = component.SimpleComponent(
     'Component', ['component'],
     ['provider', 'version', 'url', 
-     'groups', 'attributes', 'has-name', 'styles', 'events',
+     'groups', 'attributes', 'params', 'has-name', 'styles', 'events',
      'DL', 'module', 'handler',
      'menu', 'label', 'help', 'index', 'panel', 'pos', 'span'],
     specials={'provider': AttributeAttribute,
@@ -22,12 +22,14 @@ Component = component.SimpleComponent(
               'url': AttributeAttribute,
               'groups': ContentAttribute,
               'attributes': ContentAttribute,
+              'params': ContentAttribute,
               'styles': ContentAttribute,
               'events': ContentAttribute},
     params={'provider': params.ParamLongText,
             'url': params.ParamLongText,
             'groups': params.ParamContent,
             'attributes': params.ParamContent,
+            'params': params.ParamContent,
             'styles': params.ParamContent,
             'events': params.ParamContent,
             'has-name': params.ParamBool,
